@@ -7,7 +7,7 @@
 #' `update()` method installs outdated or missing packages from CRAN.
 #'
 #' @param packages A character vector of package names.
-#' @param pkgdir path to a package directory, or to a package tarball.
+#' @param pkgdir Path to a package directory, or to a package tarball.
 #' @param dependencies Which dependencies do you want to check?
 #'   Can be a character vector (selecting from "Depends", "Imports",
 #'    "LinkingTo", "Suggests", or "Enhances"), or a logical vector.
@@ -24,7 +24,7 @@
 #'   - Config/Needs/website - for dependencies used in building the pkgdown site.
 #'   - Config/Needs/coverage for dependencies used in calculating test coverage.
 #' @param quiet If `TRUE`, suppress output.
-#' @param upgrade One of "default", "ask", "always", or "never". "default"
+#' @param upgrade Should package dependencies be upgraded? One of "default", "ask", "always", or "never". "default"
 #'   respects the value of the `R_REMOTES_UPGRADE` environment variable if set,
 #'   and falls back to "ask" if unset. "ask" prompts the user for which out of
 #'   date packages to upgrade. For non-interactive sessions "ask" is equivalent
@@ -32,7 +32,8 @@
 #'   "always" and "never" respectively.
 #' @param repos A character vector giving repositories to use.
 #' @param type Type of package to `update`.
-#'
+#' @param remote_precedence A logical flag specifying whether remote sources should take precedence over
+#'   CRAN when both were found.
 #' @param object A `package_deps` object.
 #' @param ... Additional arguments passed to `install_packages`.
 #' @inheritParams install_github
@@ -122,7 +123,8 @@ local_package_deps <- function(pkgdir = ".", dependencies = NA) {
 
 dev_package_deps <- function(pkgdir = ".", dependencies = NA,
                              repos = getOption("repos"),
-                             type = getOption("pkgType")) {
+                             type = getOption("pkgType"),
+                             remote_precedence = TRUE) {
 
   pkg <- load_pkg_description(pkgdir)
   repos <- c(repos, parse_additional_repositories(pkg))
@@ -140,14 +142,14 @@ dev_package_deps <- function(pkgdir = ".", dependencies = NA,
 
   cran_deps <- package_deps(deps, repos = repos, type = type)
 
-  res <- combine_remote_deps(cran_deps, extra_deps(pkg, "remotes"))
+  res <- combine_remote_deps(cran_deps, extra_deps(pkg, "remotes"), remote_precedence)
 
   res <- do.call(rbind, c(list(res), lapply(get_extra_deps(pkg, dependencies), extra_deps, pkg = pkg), stringsAsFactors = FALSE))
 
-  res[!duplicated(res$package, fromLast = TRUE), ]
+  res[is.na(res$package) | !duplicated(res$package, fromLast = TRUE), ]
 }
 
-combine_remote_deps <- function(cran_deps, remote_deps) {
+combine_remote_deps <- function(cran_deps, remote_deps, remote_precedence) {
   # If there are no dependencies there will be no remote dependencies either,
   # so just return them (and don't force the remote_deps promise)
   if (nrow(cran_deps) == 0) {
@@ -158,7 +160,13 @@ combine_remote_deps <- function(cran_deps, remote_deps) {
   remote_deps <- remote_deps[is.na(remote_deps$package) | remote_deps$package %in% cran_deps$package, ]
 
   # If there are remote deps remove the equivalent CRAN deps
-  cran_deps <- cran_deps[!(cran_deps$package %in% remote_deps$package), ]
+  if (remote_precedence) {
+    cran_deps <- cran_deps[!(cran_deps$package %in% remote_deps$package), ]
+  # Otherwise remove remotes already covered by CRAN
+  } else {
+    remote_deps <- remote_deps[!(remote_deps$package %in% cran_deps$package), ]
+  }
+  
 
   rbind(remote_deps, cran_deps)
 }
@@ -210,7 +218,7 @@ get_extra_deps <- function(pkg, dependencies) {
   dependencies <- intersect(dependencies, names(pkg))
 
   #remove standard dependencies
-  setdiff(dependencies, tolower(standardise_dep(TRUE)))
+  setdiff(dependencies, tolower(standardise_dep(c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances"))))
 }
 
 #' @export
@@ -420,7 +428,7 @@ find_deps <- function(packages, available = available_packages(),
 #'   Any additional values that don't match one of the standard dependency
 #'   types are filtered out.
 #'
-#' @seealso <http://r-pkgs.had.co.nz/description.html#dependencies> for
+#' @seealso <https://r-pkgs.org/description.html> for
 #' additional information on what each dependency type means.
 #' @keywords internal
 #' @export
@@ -532,6 +540,18 @@ parse_one_extra <- function(x, ...) {
   } else {
     stop("Malformed remote specification '", x, "'", call. = FALSE)
   }
+
+  type = sub("^[.a-zA-Z0-9]+=", "", type)
+
+  if (grepl("@", type)) {
+    # Custom host
+    tah <- strsplit(type, "@", fixed = TRUE)[[1]]
+    type <- tah[1]
+    host <- tah[2]
+  } else {
+    host <- NULL
+  }
+
   tryCatch({
     # We need to use `environment(sys.function())` instead of
     # `asNamespace("remotes")` because when used as a script in
@@ -539,7 +559,11 @@ parse_one_extra <- function(x, ...) {
 
     fun <- get(paste0(tolower(type), "_remote"), mode = "function", inherits = TRUE)
 
-    res <- fun(repo, ...)
+    if (!is.null(host)) {
+      res <- fun(repo, host = host, ...)
+    } else {
+      res <- fun(repo, ...)
+    }
     }, error = function(e) stop("Unknown remote type: ", type, "\n  ", conditionMessage(e), call. = FALSE)
   )
   res
@@ -655,7 +679,7 @@ upgradable_packages <- function(x, upgrade, quiet, is_interactive = interactive(
   )
 }
 
-select_menu <- function(choices, title = NULL, msg = "Enter one or more numbers, or an empty line to skip updates:", width = getOption("width")) {
+select_menu <- function(choices, title = NULL, msg = "Enter one or more numbers, or an empty line to skip updates: ", width = getOption("width")) {
   if (!is.null(title)) {
     cat(title, "\n", sep = "")
   }
