@@ -359,6 +359,12 @@ test_that("remotes are parsed with explicit host", {
 
 })
 
+test_that("remotes are parsed with explicit package names", {
+  expect_equal(
+    parse_one_extra("testthat=github::hadley/testthat"),
+    github_remote("hadley/testthat"))
+})
+
 test_that("type = 'both' works well", {
 
   skip_on_cran()
